import { View, Text } from "react-native";
import React, { useEffect } from "react";
import { useDispatch, useSelector } from "react-redux";
import Database from "../../../../Components/common/DAsyncStorage";
import store from "react-native-digitcamera/src/redux/store";
import { CreateActivityId } from "react-native-digitcamera/src/Screens/OfflineFlow/CreateActivityId";
import { dlog } from "react-native-digitcamera/src/common/DLogger";
import { NamedModal } from "react-native-digitcamera/src/common/GlobalModal";

function oLog() {
  dlog("[OFFLINE ACTIVITY LISTENER] [INFO]", ...arguments);
}

const updateReduxCallback = async () => {
  const vechicleData = await Database.get("PIVechicleData");
  var registrationNumber = vechicleData?.licensePlateNumber;
  var quoteNumber = vechicleData?.quoteNumber;
  var activityTypeId = vechicleData?.activityTypeId;
  var activityId = vechicleData?.activityId;
  var toTutorial = vechicleData?.toTutorial;
  var toWithQuote = vechicleData?.toWithQuote;

  store.dispatch({
    type: "ACTIVITY_DATA",
    payload: {
      activityId: activityId,
      activityTypeId: activityTypeId,
      licensePlateNumber:
        registrationNumber === undefined ? "" : registrationNumber,
      toWithQuote: toWithQuote,
      toTutorial: toTutorial,
      quoteNumber: quoteNumber,
    },
  });
};

const addSensorDataCallback = (sensorImageDetails) => {
  store.dispatch({
    type: "SENSOR_DETAILS",
    payload: sensorImageDetails,
  });
};

const createActivity = async () => {
  const offlineVehicleData = await Database.get("PIOfflineVechicleData");
  const uploadData = offlineVehicleData.dataTobeUploaded;
  const parsedUploadData = JSON.parse(uploadData);
  oLog("parsed upload data", parsedUploadData);
  if (offlineVehicleData && parsedUploadData.length > 0) {
    oLog("[CREATING ACTIVITY HERE]");
    CreateActivityId(updateReduxCallback, addSensorDataCallback);
  } else {
    oLog("[CANNOT CREATE ACTIVITY HERE] [Empty offline data]");
  }
};

const OfflineActivityListener = () => {
  const dispatch = useDispatch();

  const network = useSelector((state) => state.network);
  const { isInternetReachable } = network;

  useEffect(() => {
    if (isInternetReachable === true) {
      // dispatch({
      //   type: "GLOBAL_MODAL_HIDE",
      // });
      oLog(" calling CreateActivityid");
      createActivity();
    } else if (isInternetReachable === false) {
      oLog(" no internet access");
    }
  }, [network]);

  return null;
};

export default OfflineActivityListener;
